@@ -1,3 +1,4 @@
+import semantic_version
 from django.conf import settings
 from drf_spectacular.utils import extend_schema_field
 from drf_spectacular.types import OpenApiTypes
@@ -10,7 +11,7 @@ from pulp_ansible.app import models
 class CollectionSerializer(serializers.ModelSerializer):
     """A serializer for a Collection."""
 
-    deprecated = serializers.BooleanField()
+    deprecated = serializers.SerializerMethodField()
     created_at = serializers.SerializerMethodField()
     updated_at = serializers.SerializerMethodField()
     href = serializers.SerializerMethodField()
@@ -31,6 +32,10 @@ class CollectionSerializer(serializers.ModelSerializer):
         )
         model = models.Collection
 
+    def get_deprecated(self, obj) -> bool:
+        """Get deprecated."""
+        return obj.pk in self.context["deprecated_collections"]
+
     def get_href(self, obj) -> str:
         """Get href."""
         return reverse(
@@ -48,29 +53,33 @@ class CollectionSerializer(serializers.ModelSerializer):
     @extend_schema_field(OpenApiTypes.DATETIME)
     def get_created_at(self, obj):
         """Get the timestamp of the lowest version CollectionVersion's created timestamp."""
-        collection = self.context["lowest_versions"][obj.pk]
-        return collection.pulp_created
+        return obj.pulp_created
 
     @extend_schema_field(OpenApiTypes.DATETIME)
     def get_updated_at(self, obj):
         """Get the timestamp of the highest version CollectionVersion's created timestamp."""
-        collection = self.context["highest_versions"][obj.pk]
-        return collection.pulp_created
+        if obj.repo_version_added_at and obj.repo_version_removed_at:
+            return max(obj.repo_version_added_at, obj.repo_version_removed_at)
+
+        return obj.repo_version_added_at or obj.repo_version_removed_at
 
     @extend_schema_field(OpenApiTypes.OBJECT)
     def get_highest_version(self, obj):
         """Get a highest version and its link."""
-        collection = self.context["highest_versions"][obj.pk]
+        available_versions = self.context["available_versions"][obj.pk]
+        version = sorted(
+            available_versions, key=lambda ver: semantic_version.Version(ver), reverse=True
+        )[0]
         href = reverse(
             "collection-versions-detail",
             kwargs={
                 "path": self.context["path"],
-                "namespace": collection.namespace,
-                "name": collection.name,
-                "version": collection.version,
+                "namespace": obj.namespace,
+                "name": obj.name,
+                "version": version,
             },
         )
-        return {"href": href, "version": str(collection.version)}
+        return {"href": href, "version": version}
 
 
 class CollectionVersionListSerializer(serializers.ModelSerializer):
@@ -79,9 +88,19 @@ class CollectionVersionListSerializer(serializers.ModelSerializer):
     href = serializers.SerializerMethodField()
     created_at = serializers.DateTimeField(source="collection.pulp_created")
     updated_at = serializers.DateTimeField(source="collection.pulp_last_updated")
+    manifest = serializers.DictField(help_text="A JSON field holding MANIFEST.json data.")
+    files = serializers.DictField(help_text="A JSON field holding FILES.json data.")
 
     class Meta:
-        fields = ("version", "href", "created_at", "updated_at")
+        fields = (
+            "version",
+            "href",
+            "created_at",
+            "updated_at",
+            "manifest",
+            "files",
+            "requires_ansible",
+        )
         model = models.CollectionVersion
 
     def get_href(self, obj) -> str:
@@ -182,7 +201,7 @@ class CollectionVersionSerializer(CollectionVersionListSerializer):
         """
         Get atrifact summary.
         """
-        return ArtifactRefSerializer(self.context["content_artifact"]).data
+        return ArtifactRefSerializer(obj.contentartifact_set.get()).data
 
     def get_download_url(self, obj) -> str:
         """
@@ -190,7 +209,7 @@ class CollectionVersionSerializer(CollectionVersionListSerializer):
         """
         host = settings.ANSIBLE_CONTENT_HOSTNAME.strip("/")
         distro_base_path = self.context["path"]
-        filename_path = self.context["content_artifact"].relative_path.lstrip("/")
+        filename_path = obj.contentartifact_set.get().relative_path.lstrip("/")
         download_url = f"{host}/{distro_base_path}/{filename_path}"
         return download_url
 
@@ -203,3 +222,13 @@ class CollectionVersionDocsSerializer(serializers.ModelSerializer):
     class Meta:
         fields = ("docs_blob",)
         model = models.CollectionVersion
+
+
+class RepoMetadataSerializer(serializers.ModelSerializer):
+    """A serializer to display RepositoryVersion metadata."""
+
+    published = serializers.DateTimeField(source="pulp_created")
+
+    class Meta:
+        fields = ("published",)
+        model = models.RepositoryVersion
