@@ -1,15 +1,16 @@
-from collections import defaultdict, namedtuple
 from datetime import datetime
 from gettext import gettext as _
 import semantic_version
 
-from django.db.models import Exists, OuterRef
+from django.contrib.postgres.aggregates import ArrayAgg
+from django.db.models import F
+from django.db.models.expressions import Window
+from django.db.models.functions.window import FirstValue
 from django.shortcuts import get_object_or_404
-from django.utils.datastructures import MultiValueDictKeyError
 from django.utils.dateparse import parse_datetime
+from django_filters import filters
 from drf_spectacular.utils import OpenApiParameter, extend_schema
 from rest_framework import mixins
-from rest_framework.exceptions import ValidationError
 from rest_framework.response import Response
 from rest_framework.reverse import reverse
 from rest_framework import serializers
@@ -17,8 +18,9 @@ from rest_framework import status as http_status
 from rest_framework import viewsets
 
 from pulpcore.plugin.exceptions import DigestValidationError
-from pulpcore.plugin.models import PulpTemporaryFile, Content, ContentArtifact
+from pulpcore.plugin.models import PulpTemporaryFile, Content
 from pulpcore.plugin.serializers import AsyncOperationResponseSerializer
+from pulpcore.plugin.viewsets import BaseFilterSet
 
 from pulp_ansible.app.galaxy.v3.exceptions import ExceptionHandlerMixin
 from pulp_ansible.app.galaxy.v3.serializers import (
@@ -26,6 +28,7 @@ from pulp_ansible.app.galaxy.v3.serializers import (
     CollectionVersionSerializer,
     CollectionVersionDocsSerializer,
     CollectionVersionListSerializer,
+    RepoMetadataSerializer,
 )
 from pulp_ansible.app.models import (
     AnsibleCollectionDeprecated,
@@ -41,10 +44,7 @@ from pulp_ansible.app.serializers import (
 
 from pulp_ansible.app.galaxy.mixins import UploadGalaxyCollectionMixin
 from pulp_ansible.app.galaxy.v3.pagination import LimitOffsetPagination
-from pulp_ansible.app.viewsets import CollectionFilter, CollectionVersionFilter
-
-
-CollectionTuple = namedtuple("CollectionTuple", ["namespace", "name", "version", "pulp_created"])
+from pulp_ansible.app.viewsets import CollectionVersionFilter
 
 
 class AnsibleDistributionMixin:
@@ -52,19 +52,27 @@ class AnsibleDistributionMixin:
     A mixin for ViewSets that use AnsibleDistribution.
     """
 
-    @staticmethod
-    def get_repository_version(path):
+    @property
+    def _repository_version(self):
         """Returns repository version."""
+        path = self.kwargs["path"]
+        context = getattr(self, "pulp_context", None)
+        if context and context.get(path, None):
+            return self.pulp_context[path]
+
         distro = get_object_or_404(AnsibleDistribution, base_path=path)
         if distro.repository_version:
+            self.pulp_context = {path: distro.repository_version}
             return distro.repository_version
 
-        return distro.repository.latest_version()
+        repo_version = distro.repository.latest_version()
+        self.pulp_context = {path: repo_version}
+        return repo_version
 
-    @staticmethod
-    def get_distro_content(path):
+    @property
+    def _distro_content(self):
         """Returns distribution content."""
-        repo_version = AnsibleDistributionMixin.get_repository_version(path)
+        repo_version = self._repository_version
         if repo_version is None:
             return Content.objects.none()
 
@@ -87,9 +95,11 @@ class CollectionVersionRetrieveMixin:
         """
         Returns a CollectionVersions queryset for specified distribution.
         """
-        distro_content = self.get_distro_content(self.kwargs["path"])
+        distro_content = self._distro_content
 
-        collections = CollectionVersion.objects.select_related("collection").filter(
+        collections = CollectionVersion.objects.select_related(
+            "content_ptr__contentartifact"
+        ).filter(
             pk__in=distro_content, namespace=self.kwargs["namespace"], name=self.kwargs["name"]
         )
         return collections
@@ -99,16 +109,38 @@ class CollectionVersionRetrieveMixin:
         Returns a CollectionVersion object.
         """
         instance = self.get_object()
-        artifact = ContentArtifact.objects.get(content=instance)
 
         context = self.get_serializer_context()
-        context["content_artifact"] = artifact
 
         serializer = self.get_serializer_class()(instance, context=context)
 
         return Response(serializer.data)
 
 
+class CollectionFilter(BaseFilterSet):
+    """
+    FilterSet for Ansible Collections.
+    """
+
+    namespace = filters.CharFilter(field_name="namespace")
+    name = filters.CharFilter(field_name="name")
+    deprecated = filters.BooleanFilter(field_name="deprecated", method="get_deprecated")
+
+    def get_deprecated(self, qs, name, value):
+        """Deprecated filter."""
+        deprecation = self.request.parser_context["view"]._deprecation
+        if value:
+            return qs.filter(pk__in=deprecation)
+
+        if value is False:
+            return qs.exclude(pk__in=deprecation)
+        return qs
+
+    class Meta:
+        model = Collection
+        fields = ["namespace", "name", "deprecated"]
+
+
 class CollectionViewSet(
     ExceptionHandlerMixin,
     AnsibleDistributionMixin,
@@ -126,68 +158,107 @@ class CollectionViewSet(
     filterset_class = CollectionFilter
     pagination_class = LimitOffsetPagination
 
+    @property
+    def _deprecation(self):
+        """Return deprecated collecion ids."""
+        repo_version = self._repository_version
+        deprecated = AnsibleCollectionDeprecated.objects.filter(
+            repository_version=repo_version
+        ).values_list("collection_id", flat=True)
+        self.deprecated_collections_context = deprecated  # needed by get__serializer_context
+        return deprecated
+
+    def get_queryset(self):
+        """
+        Returns a Collections queryset for specified distribution.
+        """
+        repo_version = self._repository_version
+        return Collection.objects.filter(
+            versions__in=repo_version.content,
+        )
+
+    def append_context(self, queryset):
+        """Appending collection data to context."""
+        repo_version = self._repository_version
+        collections_qs = queryset.annotate(available_versions=ArrayAgg("versions__version"))
+        versions_context = {}
+        for collection_id, available_versions in collections_qs.values_list(
+            "pk", "available_versions"
+        ):
+            versions_context[collection_id] = available_versions
+
+        self.available_versions_context = versions_context  # needed by get__serializer_context
+        self._deprecation
+
+        collections = Collection.objects.filter(
+            pk__in=versions_context.keys(),
+            versions__version_memberships__repository=repo_version.repository,
+        ).annotate(
+            repo_version_added_at=Window(
+                expression=FirstValue(
+                    "versions__version_memberships__version_added__pulp_last_updated"
+                ),
+                partition_by=[F("versions__collection_id")],
+                order_by=F("versions__version_memberships__version_added__pulp_last_updated").desc(
+                    nulls_last=True
+                ),
+            ),
+            repo_version_removed_at=Window(
+                expression=FirstValue(
+                    "versions__version_memberships__version_removed__pulp_last_updated"
+                ),
+                partition_by=[F("versions__collection_id")],
+                order_by=F(
+                    "versions__version_memberships__version_removed__pulp_last_updated"
+                ).desc(nulls_last=True),
+            ),
+        )
+
+        return collections.distinct("versions__collection_id").only(
+            "pulp_created", "name", "namespace"
+        )
+
     def filter_queryset(self, queryset):
         """
         Filter Repository related fields.
         """
         queryset = super().filter_queryset(queryset)
 
-        try:
-            user_deprecated_value = self.request.query_params["deprecated"]
-        except MultiValueDictKeyError:
-            pass
-        else:
-            if user_deprecated_value.lower() in ["true", "yes", "1"]:
-                queryset = queryset.filter(deprecated=True)
-            elif user_deprecated_value.lower() in ["false", "no", "0"]:
-                queryset = queryset.filter(deprecated=False)
-            else:
-                raise ValidationError("Cannot parse value of `deprecated` GET parameter")
+        if self.paginator is None:
+            queryset = self.append_context(queryset)
 
         return queryset
 
-    def get_queryset(self):
-        """
-        Returns a Collections queryset for specified distribution.
-        """
-        repo_version = self.get_repository_version(self.kwargs["path"])
-        deprecated_query = AnsibleCollectionDeprecated.objects.filter(
-            collection=OuterRef("pk"), repository_version=repo_version
+    def paginate_queryset(self, queryset):
+        """Custom pagination."""
+        if self.paginator is None:
+            return None
+        paginator = self.paginator
+        # Making sure COUNT a lighter query (before append_context)
+        paginator.count = paginator.get_count(
+            queryset.distinct("versions__collection_id").only("pk")
         )
-        collections = Collection.objects.filter(versions__in=repo_version.content).distinct()
-        collections = collections.annotate(deprecated=Exists(deprecated_query))
+        paginator.limit = paginator.get_limit(self.request)
+        if paginator.limit is None:
+            return None
 
-        versions_qs = CollectionVersion.objects.filter(pk__in=repo_version.content).values_list(
-            "collection_id",
-            "namespace",
-            "name",
-            "version",
-            "pulp_created",
-        )
+        paginator.offset = paginator.get_offset(self.request)
+        paginator.request = self.request
+        if paginator.count > paginator.limit and paginator.template is not None:
+            paginator.display_page_controls = True
 
-        highest_versions = defaultdict(
-            lambda: CollectionTuple(None, None, semantic_version.Version("0.0.0"), None)
-        )
-        lowest_versions = defaultdict(
-            lambda: CollectionTuple(None, None, semantic_version.Version("100000000000.0.0"), None)
-        )
-        for collection_id, namespace, name, version, pulp_created in versions_qs:
-            version_to_consider = semantic_version.Version(version)
-            collection_tuple = CollectionTuple(namespace, name, version_to_consider, pulp_created)
-            if version_to_consider > highest_versions[collection_id].version:
-                highest_versions[collection_id] = collection_tuple
-            if version_to_consider < lowest_versions[collection_id].version:
-                lowest_versions[collection_id] = collection_tuple
-
-        self.highest_versions_context = highest_versions  # needed by get__serializer_context
-        self.lowest_versions_context = lowest_versions  # needed by get__serializer_context
-        return collections
+        if paginator.count == 0 or paginator.offset > paginator.count:
+            return []
+        new_queryset = queryset[paginator.offset : paginator.offset + paginator.limit]
+        # Paginate query with appended context
+        return list(self.append_context(new_queryset))
 
     def get_object(self):
         """
         Returns a Collection object.
         """
         queryset = self.filter_queryset(self.get_queryset())
+        queryset = self.append_context(queryset)
         return get_object_or_404(
             queryset, namespace=self.kwargs["namespace"], name=self.kwargs["name"]
         )
@@ -199,19 +270,20 @@ class CollectionViewSet(
         This uses super() but also adds in the "highest_versions" data from get_queryset()
         """
         super_data = super().get_serializer_context()
-        super_data["highest_versions"] = self.highest_versions_context
-        super_data["lowest_versions"] = self.lowest_versions_context
+        if getattr(self, "available_versions_context", None):
+            super_data["available_versions"] = self.available_versions_context
+        super_data["deprecated_collections"] = self.deprecated_collections_context
         return super_data
 
     def update(self, request, *args, **kwargs):
         """
         Update a Collection object.
         """
-        repo_version = self.get_repository_version(self.kwargs["path"])
+        repo_version = self._repository_version
         collection = self.get_object()
         serializer = self.get_serializer(collection, data=request.data, partial=True)
         serializer.is_valid(raise_exception=True)
-        deprecated_value = serializer.validated_data["deprecated"]
+        deprecated_value = request.data.get("deprecated")
         if deprecated_value:
             AnsibleCollectionDeprecated.objects.get_or_create(
                 repository_version=repo_version, collection=collection
@@ -223,6 +295,12 @@ class CollectionViewSet(
         return Response(serializer.data)
 
 
+class UnpaginatedCollectionViewSet(CollectionViewSet):
+    """Unpaginated ViewSet for Collections."""
+
+    pagination_class = None
+
+
 class CollectionUploadViewSet(
     ExceptionHandlerMixin, viewsets.GenericViewSet, UploadGalaxyCollectionMixin
 ):
@@ -327,6 +405,40 @@ class CollectionVersionViewSet(
         return Response(serializer.data)
 
 
+class UnpaginatedCollectionVersionViewSet(CollectionVersionViewSet):
+    """Unpaginated ViewSet for CollectionVersions."""
+
+    pagination_class = None
+
+    def get_queryset(self):
+        """
+        Returns a CollectionVersions queryset for specified distribution.
+        """
+        distro_content = self._distro_content
+
+        return CollectionVersion.objects.select_related("content_ptr__contentartifact").filter(
+            pk__in=distro_content
+        )
+
+    def list(self, request, *args, **kwargs):
+        """
+        Returns paginated CollectionVersions list.
+        """
+        queryset = self.filter_queryset(self.get_queryset())
+        queryset = sorted(
+            queryset, key=lambda obj: semantic_version.Version(obj.version), reverse=True
+        )
+
+        context = self.get_serializer_context()
+        page = self.paginate_queryset(queryset)
+        if page is not None:
+            serializer = CollectionVersionSerializer(page, many=True, context=context)
+            return self.get_paginated_response(serializer.data)
+
+        serializer = CollectionVersionSerializer(queryset, many=True, context=context)
+        return Response(serializer.data)
+
+
 class CollectionVersionDocsViewSet(
     CollectionVersionRetrieveMixin,
     ExceptionHandlerMixin,
@@ -382,3 +494,24 @@ class CollectionImportViewSet(
         serializer = CollectionImportDetailSerializer(instance, context=context)
 
         return Response(serializer.data)
+
+
+class RepoMetadataViewSet(
+    ExceptionHandlerMixin,
+    AnsibleDistributionMixin,
+    mixins.RetrieveModelMixin,
+    viewsets.GenericViewSet,
+):
+    """
+    ViewSet for Repository Metadata.
+    """
+
+    authentication_classes = []
+    permission_classes = []
+    serializer_class = RepoMetadataSerializer
+
+    def get_object(self):
+        """
+        Returns a RepositoryVersion object.
+        """
+        return self._repository_version
