@@ -1,15 +1,18 @@
 import asyncio
-from aiohttp.client_exceptions import ClientResponseError
+from collections import defaultdict
 from gettext import gettext as _
 import json
 import logging
 import tarfile
+import yaml
 from urllib.parse import urljoin
 
+from aiohttp.client_exceptions import ClientResponseError
 from async_lru import alru_cache
 from django.db import transaction
 from django.db.models import Q
 from django.urls import reverse
+from django.utils.dateparse import parse_datetime
 from galaxy_importer.collection import import_collection as process_collection
 from galaxy_importer.collection import CollectionFilename
 from galaxy_importer.exceptions import ImporterError
@@ -50,8 +53,9 @@ from pulp_ansible.app.models import (
     CollectionVersion,
     Tag,
 )
-from pulp_ansible.app.serializers import CollectionVersionSerializer
+from pulp_ansible.app.serializers import CollectionVersionSerializer, CollectionRemoteSerializer
 from pulp_ansible.app.tasks.utils import (
+    get_file_obj_from_tarball,
     parse_metadata,
     parse_collections_requirements_file,
     RequirementsFileEntry,
@@ -61,7 +65,45 @@ from pulp_ansible.app.tasks.utils import (
 log = logging.getLogger(__name__)
 
 
-def sync(remote_pk, repository_pk, mirror):
+def update_collection_remote(remote_pk, *args, **kwargs):
+    """
+    Update a CollectionRemote.
+
+    Update `AnsibleRepository.last_synced_metadata_time` when URL or requirements file are updated.
+
+    Args:
+        remote_pk (str): the id of the CollectionRemote
+        data (dict): dictionary whose keys represent the fields of the model and their corresponding
+            values.
+        partial (bool): When true, only the fields specified in the data dictionary are updated.
+            When false, any fields missing from the data dictionary are assumed to be None and
+            their values are updated as such.
+
+    Raises:
+        :class:`rest_framework.exceptions.ValidationError`: When serializer instance can't be saved
+            due to validation error. This theoretically should never occur since validation is
+            performed before the task is dispatched.
+    """
+    data = kwargs.pop("data", None)
+    partial = kwargs.pop("partial", False)
+    with transaction.atomic():
+        if "url" in data or "requirements_file" in data:
+            repos = list(
+                AnsibleRepository.objects.filter(
+                    remote_id=remote_pk, last_synced_metadata_time__isnull=False
+                ).all()
+            )
+            for repo in repos:
+                repo.last_synced_metadata_time = None
+            AnsibleRepository.objects.bulk_update(repos, ["last_synced_metadata_time"])
+
+        instance = CollectionRemote.objects.get(pk=remote_pk)
+        serializer = CollectionRemoteSerializer(instance, data=data, partial=partial)
+        serializer.is_valid(raise_exception=True)
+        serializer.save()
+
+
+def sync(remote_pk, repository_pk, mirror, optimize):
     """
     Sync Collections with ``remote_pk``, and save a new RepositoryVersion for ``repository_pk``.
 
@@ -69,6 +111,7 @@ def sync(remote_pk, repository_pk, mirror):
         remote_pk (str): The remote PK.
         repository_pk (str): The repository PK.
         mirror (bool): True for mirror mode, False for additive.
+        optimize (boolean): Whether to optimize sync or not.
 
     Raises:
         ValueError: If the remote does not specify a URL to sync.
@@ -76,15 +119,23 @@ def sync(remote_pk, repository_pk, mirror):
     """
     remote = CollectionRemote.objects.get(pk=remote_pk)
     repository = AnsibleRepository.objects.get(pk=repository_pk)
+    latest_version_before_sync = repository.latest_version()
 
     if not remote.url:
         raise ValueError(_("A CollectionRemote must have a 'url' specified to synchronize."))
 
-    first_stage = CollectionSyncFirstStage(remote)
+    first_stage = CollectionSyncFirstStage(remote, repository, optimize)
     d_version = AnsibleDeclarativeVersion(first_stage, repository, mirror=mirror)
     d_version.create()
 
     repo_version = repository.latest_version()
+    if repo_version.number == latest_version_before_sync.number:
+        return
+
+    if first_stage.last_synced_metadata_time:
+        repository.last_synced_metadata_time = first_stage.last_synced_metadata_time
+        repository.save()
+
     to_deprecate = []
     if first_stage.deprecations:
         collections_deprecate_true_qs = Collection.objects.filter(first_stage.deprecations)
@@ -203,6 +254,7 @@ def create_collection_from_importer(importer_result):
         collection_version = CollectionVersion(
             collection=collection,
             **collection_info,
+            requires_ansible=importer_result.get("requires_ansible"),
             contents=importer_result["contents"],
             docs_blob=importer_result["docs_blob"],
         )
@@ -290,28 +342,36 @@ class CollectionSyncFirstStage(Stage):
     The first stage of a pulp_ansible sync pipeline.
     """
 
-    def __init__(self, remote):
+    def __init__(self, remote, repository, optimize):
         """
         The first stage of a pulp_ansible sync pipeline.
 
         Args:
             remote (CollectionRemote): The remote data to be used when syncing
+            repository (AnsibleRepository): The repository being syncedself.
+            optimize (boolean): Whether to optimize sync or not.
 
         """
         super().__init__()
         msg = _("Parsing CollectionVersion Metadata")
         self.parsing_metadata_progress_bar = ProgressReport(message=msg, code="parsing.metadata")
         self.remote = remote
+        self.repository = repository
+        self.optimize = optimize
         self.collection_info = parse_collections_requirements_file(remote.requirements_file)
         self.deprecations = Q()
 
+        self._unpaginated_collection_metadata = None
+        self._unpaginated_collection_version_metadata = None
+        self.last_synced_metadata_time = None
+
         # Interpret download policy
         self.deferred_download = self.remote.policy != Remote.IMMEDIATE
 
     @alru_cache(maxsize=128)
-    async def _get_collection_api(self, root):
+    async def _get_root_api(self, root):
         """
-        Returns the collection api path and api version.
+        Returns the root api path and api version.
 
         Based on https://git.io/JTMxE.
         """
@@ -340,14 +400,27 @@ class CollectionSyncFirstStage(Stage):
         else:
             raise RuntimeError(_("Unsupported API versions at {}").format(root))
 
-        endpoint = f"{root}v{api_version}/collections/"
+        endpoint = f"{root}v{api_version}"
 
         return endpoint, api_version
 
+    @alru_cache(maxsize=128)
+    async def _get_paginated_collection_api(self, root):
+        """
+        Returns the collection api path and api version.
+
+        Based on https://git.io/JTMxE.
+        """
+        endpoint, api_version = await self._get_root_api(root)
+        return f"{endpoint}/collections/", api_version
+
     async def _fetch_collection_version_metadata(self, api_version, collection_version_url):
         downloader = self.remote.get_downloader(url=collection_version_url)
         metadata = parse_metadata(await downloader.run())
+        await self._add_collection_version(api_version, collection_version_url, metadata)
 
+    async def _add_collection_version(self, api_version, collection_version_url, metadata):
+        """Add CollectionVersion to the sync pipeline."""
         url = metadata["download_url"]
 
         collection_version = CollectionVersion(
@@ -397,17 +470,9 @@ class CollectionSyncFirstStage(Stage):
             versions_list_url = f"{url_without_get_params}?limit={page_size}&offset={offset}"
         return self.remote.get_downloader(url=versions_list_url)
 
-    async def _fetch_collection_metadata(self, requirements_entry):
-        if requirements_entry.version == "*":
-            requirement_version = Requirement.parse("collection")
-        else:
-            requirement_version = Requirement.parse(f"collection{requirements_entry.version}")
-        if requirements_entry.source:
-            root = requirements_entry.source
-        else:
-            root = self.remote.url
-        collection_endpoint, api_version = await self._get_collection_api(root)
-        namespace, name = requirements_entry.name.split(".")
+    async def _fetch_paginated_collection_metadata(self, name, namespace, requirement, source=None):
+        root = source or self.remote.url
+        collection_endpoint, api_version = await self._get_paginated_collection_api(root)
         collection_url = f"{collection_endpoint}{namespace}/{name}"
         collection_metadata_downloader = self.remote.get_downloader(url=collection_url)
         collection_metadata = parse_metadata(await collection_metadata_downloader.run())
@@ -425,7 +490,7 @@ class CollectionSyncFirstStage(Stage):
             else:
                 collection_versions = collection_versions_list["data"]
             for collection_version in collection_versions:
-                if collection_version["version"] in requirement_version:
+                if collection_version["version"] in requirement:
                     version_num = collection_version["version"]
                     collection_version_detail_url = f"{collection_url}/versions/{version_num}/"
                     if collection_metadata["deprecated"]:
@@ -445,6 +510,41 @@ class CollectionSyncFirstStage(Stage):
 
         await asyncio.gather(*tasks)
 
+    async def _read_from_downloaded_metadata(self, name, namespace, requirement):
+        tasks = []
+        loop = asyncio.get_event_loop()
+
+        if self._unpaginated_collection_metadata[namespace][name]["deprecated"]:
+            self.deprecations |= Q(namespace=namespace, name=name)
+
+        all_versions_of_collection = self._unpaginated_collection_version_metadata[namespace][name]
+
+        for col_version_metadata in all_versions_of_collection:
+            if col_version_metadata["version"] in requirement:
+                collection_version_url = urljoin(self.remote.url, f"{col_version_metadata['href']}")
+                tasks.append(
+                    loop.create_task(
+                        self._add_collection_version(
+                            self._api_version, collection_version_url, col_version_metadata
+                        )
+                    )
+                )
+
+    async def _fetch_collection_metadata(self, requirements_entry):
+        if requirements_entry.version == "*":
+            requirement_version = Requirement.parse("collection")
+        else:
+            requirement_version = Requirement.parse(f"collection{requirements_entry.version}")
+
+        namespace, name = requirements_entry.name.split(".")
+
+        if self._unpaginated_collection_version_metadata and requirements_entry.source is None:
+            await self._read_from_downloaded_metadata(name, namespace, requirement_version)
+        else:
+            await self._fetch_paginated_collection_metadata(
+                name, namespace, requirement_version, requirements_entry.source
+            )
+
     @staticmethod
     def _get_response_next_value(api_version, response):
         if api_version == 2:
@@ -460,8 +560,72 @@ class CollectionSyncFirstStage(Stage):
             collection_list_url = f"{collection_endpoint}?limit={page_size}&offset={offset}"
         return self.remote.get_downloader(url=collection_list_url)
 
+    async def _download_unpaginated_metadata(self):
+        root_endpoint, api_version = await self._get_root_api(self.remote.url)
+        self._api_version = api_version
+        if api_version > 2:
+            collection_endpoint = f"{root_endpoint}/collections/all/"
+            downloader = self.remote.get_downloader(
+                url=collection_endpoint, silence_errors_for_response_status_codes={404}
+            )
+            try:
+                collection_metadata_list = parse_metadata(await downloader.run())
+            except FileNotFoundError:
+                pass
+            else:
+                self._unpaginated_collection_metadata = defaultdict(dict)
+                for collection in collection_metadata_list:
+                    namespace = collection["namespace"]
+                    name = collection["name"]
+                    self._unpaginated_collection_metadata[namespace][name] = collection
+
+                collection_version_endpoint = f"{root_endpoint}/collection_versions/all/"
+                downloader = self.remote.get_downloader(url=collection_version_endpoint)
+                collection_version_metadata_list = parse_metadata(await downloader.run())
+
+                self._unpaginated_collection_version_metadata = defaultdict(
+                    lambda: defaultdict(list)
+                )
+                for collection_version_metadata in collection_version_metadata_list:
+                    namespace = collection_version_metadata["namespace"]["name"]
+                    name = collection_version_metadata["name"]
+                    self._unpaginated_collection_version_metadata[namespace][name].append(
+                        collection_version_metadata
+                    )
+
+    async def _find_all_collections_from_unpaginated_data(self):
+        tasks = []
+        loop = asyncio.get_event_loop()
+
+        for collection_namespace_dict in self._unpaginated_collection_metadata.values():
+            for collection in collection_namespace_dict.values():
+                if collection["deprecated"]:
+                    self.deprecations |= Q(
+                        namespace=collection["namespace"], name=collection["name"]
+                    )
+
+        for collections_in_namespace in self._unpaginated_collection_version_metadata.values():
+            for collection_versions in collections_in_namespace.values():
+                for collection_version in collection_versions:
+                    collection_version_url = urljoin(
+                        self.remote.url, f"{collection_version['href']}"
+                    )
+                    tasks.append(
+                        loop.create_task(
+                            self._add_collection_version(
+                                self._api_version, collection_version_url, collection_version
+                            )
+                        )
+                    )
+
+        await asyncio.gather(*tasks)
+
     async def _find_all_collections(self):
-        collection_endpoint, api_version = await self._get_collection_api(self.remote.url)
+        if self._unpaginated_collection_version_metadata:
+            await self._find_all_collections_from_unpaginated_data()
+            return
+
+        collection_endpoint, api_version = await self._get_paginated_collection_api(self.remote.url)
         loop = asyncio.get_event_loop()
 
         tasks = []
@@ -497,12 +661,67 @@ class CollectionSyncFirstStage(Stage):
 
         await asyncio.gather(*tasks)
 
+    async def _should_we_sync(self):
+        """Check last synced metadata time."""
+        msg = _("no-op: Checking if remote changed since last sync.")
+        noop = ProgressReport(message=msg, code="noop")
+        noop.state = TASK_STATES.COMPLETED
+        noop.save()
+
+        if not self.repository.remote:
+            return True
+
+        if self.remote != self.repository.remote.cast():
+            return True
+
+        root, api_version = await self._get_root_api(self.remote.url)
+        if api_version == 3:
+            downloader = self.remote.get_downloader(
+                url=root, silence_errors_for_response_status_codes={404}
+            )
+            try:
+                metadata = parse_metadata(await downloader.run())
+            except FileNotFoundError:
+                return True
+
+            try:
+                self.last_synced_metadata_time = parse_datetime(metadata["published"])
+            except KeyError:
+                return True
+
+            sources = set()
+            if self.collection_info:
+                sources = {r.source for r in self.collection_info if r.source}
+            sources.add(self.remote.url)
+            if len(sources) > 1:
+                return True
+
+            if self.last_synced_metadata_time == self.repository.last_synced_metadata_time:
+                noop.message = _(
+                    "no-op: {remote} did not change since last sync - {published}".format(
+                        remote=self.remote.url, published=self.last_synced_metadata_time
+                    )
+                )
+                noop.save()
+                return False
+
+        return True
+
     async def run(self):
         """
         Build and emit `DeclarativeContent` from the ansible metadata.
         """
+        if self.optimize:
+            should_we_sync = await self._should_we_sync()
+            if should_we_sync is False:
+                log.debug(_("no-op: remote wasn't updated since last sync."))
+                return
+
         tasks = []
         loop = asyncio.get_event_loop()
+
+        await self._download_unpaginated_metadata()
+
         if self.collection_info:
             for requirement_entry in self.collection_info:
                 tasks.append(loop.create_task(self._fetch_collection_metadata(requirement_entry)))
@@ -611,17 +830,20 @@ class CollectionContentSaver(ContentSaver):
                 with artifact.file.open() as artifact_file, tarfile.open(
                     fileobj=artifact_file, mode="r"
                 ) as tar:
-                    log.info(_("Reading MANIFEST.json from {path}").format(path=artifact.file.name))
-                    for manifest in ["MANIFEST.json", "./MANIFEST.json"]:
-                        try:
-                            file_obj = tar.extractfile(manifest)
-                        except KeyError:
-                            file_obj = None
-                        else:
-                            break
-                    if not file_obj:
-                        raise FileNotFoundError("MANIFEST.json not found")
-                    manifest_data = json.load(file_obj)
+                    runtime_metadata = get_file_obj_from_tarball(
+                        tar, "meta/runtime.yml", artifact.file.name, raise_exc=False
+                    )
+                    if runtime_metadata:
+                        runtime_yaml = yaml.safe_load(runtime_metadata)
+                        collection_version.requires_ansible = runtime_yaml.get("requires_ansible")
+                    manifest_data = json.load(
+                        get_file_obj_from_tarball(tar, "MANIFEST.json", artifact.file.name)
+                    )
+                    files_data = json.load(
+                        get_file_obj_from_tarball(tar, "FILES.json", artifact.file.name)
+                    )
+                    collection_version.manifest = manifest_data
+                    collection_version.files = files_data
                     info = manifest_data["collection_info"]
 
                 # Create the tags
